@@ -12,6 +12,7 @@
 #include "orcus/types.hpp"
 
 #include <string>
+#include <memory>
 
 namespace orcus {
 
@@ -22,7 +23,7 @@ namespace iface {
 class ORCUS_DLLPUBLIC import_filter
 {
     struct impl;
-    impl* mp_impl;
+    std::unique_ptr<impl> mp_impl;
 
 public:
     import_filter(format_t input);
@@ -44,10 +45,7 @@ class ORCUS_DLLPUBLIC document_dumper
 {
 public:
     virtual ~document_dumper();
-    virtual void dump_flat(const std::string& outdir) const = 0;
-    virtual void dump_html(const std::string& outdir) const = 0;
-    virtual void dump_json(const std::string& outdir) const = 0;
-    virtual void dump_csv(const std::string& outdir) const = 0;
+    virtual void dump(dump_format_t format, const std::string& output) const = 0;
     virtual void dump_check(std::ostream& os) const = 0;
 };
 
