@@ -59,13 +59,74 @@ public:
      */
     virtual size_t add(const char* s, size_t n) = 0;
 
+    /**
+     * Set the index of a font to apply to the current format attributes.
+     *
+     * @param font_index positive integer representing the font to use.
+     */
     virtual void set_segment_font(size_t font_index) = 0;
+
+    /**
+     * Set whether or not to make the font bold to the current format
+     * attributes.
+     *
+     * @param b true if it's bold, false otherwise.
+     */
     virtual void set_segment_bold(bool b) = 0;
+
+    /**
+     * Set whether or not to set the font italic font to the current format
+     * attributes.
+     *
+     * @param b true if it's italic, false otherwise.
+     */
     virtual void set_segment_italic(bool b) = 0;
+
+    /**
+     * Set the name of a font to the current format attributes.
+     *
+     * @param s pointer to the first character of a char array that stores the
+     *          font name.
+     * @param n size of the char array that stores the font name.
+     */
     virtual void set_segment_font_name(const char* s, size_t n) = 0;
+
+    /**
+     * Set a font size to the current format attributes.
+     *
+     * @param point font size in points.
+     */
     virtual void set_segment_font_size(double point) = 0;
+
+    /**
+     * Set the color of a font in ARGB to the current format attributes.
+     *
+     * @param alpha alpha component value (0-255).
+     * @param red red component value (0-255).
+     * @param green green component value (0-255).
+     * @param blue blue component value (0-255).
+     */
     virtual void set_segment_font_color(color_elem_t alpha, color_elem_t red, color_elem_t green, color_elem_t blue) = 0;
+
+    /**
+     * Append a string segment with the current format attributes to the
+     * formatted string buffer.
+     *
+     * @param s pointer to the first character of the string array.  The
+     *          string array doesn't necessary have to be null-terminated.
+     * @param n length of the string.
+     */
     virtual void append_segment(const char* s, size_t n) = 0;
+
+    /**
+     * Store the formatted string in the current buffer to the shared strings
+     * store.  The implementation may choose to unconditionally append the
+     * string to the store, or choose to look for an existing indentical
+     * formatted string to reuse and discard the new one if one exists.
+     *
+     * @return ID of the string just inserted, or the ID of an existing string
+     *         with identical formatting attributes.
+     */
     virtual size_t commit_segments() = 0;
 };
 
@@ -93,34 +154,75 @@ public:
     virtual void set_font_italic(bool b) = 0;
     virtual void set_font_name(const char* s, size_t n) = 0;
     virtual void set_font_size(double point) = 0;
-    virtual void set_font_underline(orcus::spreadsheet::underline_t e) = 0;
+    virtual void set_font_underline(underline_t e) = 0;
     virtual void set_font_underline_width(underline_width_t e) = 0;
     virtual void set_font_underline_mode(underline_mode_t e) = 0;
     virtual void set_font_underline_type(underline_type_t e) = 0;
     virtual void set_font_underline_color(color_elem_t alpha, color_elem_t red, color_elem_t green, color_elem_t blue) = 0;
     virtual void set_font_color(color_elem_t alpha, color_elem_t red, color_elem_t green, color_elem_t blue) = 0;
-    virtual void set_strikethrough_style(orcus::spreadsheet::strikethrough_style_t s) = 0;
-    virtual void set_strikethrough_type(orcus::spreadsheet::strikethrough_type_t s) = 0;
-    virtual void set_strikethrough_width(orcus::spreadsheet::strikethrough_width_t s) = 0;
-    virtual void set_strikethrough_text(orcus::spreadsheet::strikethrough_text_t s) = 0;
+    virtual void set_strikethrough_style(strikethrough_style_t s) = 0;
+    virtual void set_strikethrough_type(strikethrough_type_t s) = 0;
+    virtual void set_strikethrough_width(strikethrough_width_t s) = 0;
+    virtual void set_strikethrough_text(strikethrough_text_t s) = 0;
     virtual size_t commit_font() = 0;
 
     // fill
 
+    /**
+     * Set the total number of fill styles.  This call is not strictly
+     * required but may slightly improve performance.
+     *
+     * @param n number of fill styles.
+     */
     virtual void set_fill_count(size_t n) = 0;
-    virtual void set_fill_pattern_type(const char* s, size_t n) = 0;
+
+    /**
+     * Set the type of fill pattern.
+     *
+     * @param fp fill pattern type.
+     */
+    virtual void set_fill_pattern_type(fill_pattern_t fp) = 0;
+
+    /**
+     * Set the foreground color of a fill.  <i>Note that for a solid fill
+     * type, the foreground color will be used.</i>
+     *
+     * @param alpha alpha component ranging from 0 (fully transparent) to 255
+     *              (fully opaque).
+     * @param red red component ranging from 0 to 255.
+     * @param green green component ranging from 0 to 255.
+     * @param blue blue component ranging from 0 to 255.
+     */
     virtual void set_fill_fg_color(color_elem_t alpha, color_elem_t red, color_elem_t green, color_elem_t blue) = 0;
+
+    /**
+     * Set the background color of a fill.  <i>Note that this color will
+     * be ignored for a solid fill type.</i>
+     *
+     * @param alpha alpha component ranging from 0 (fully transparent) to 255
+     *              (fully opaque).
+     * @param red red component ranging from 0 to 255.
+     * @param green green component ranging from 0 to 255.
+     * @param blue blue component ranging from 0 to 255.
+     */
     virtual void set_fill_bg_color(color_elem_t alpha, color_elem_t red, color_elem_t green, color_elem_t blue) = 0;
+
+    /**
+     * Commit the fill style currently in the buffer.
+     *
+     * @return the ID of the committed fill style, to be passed on to the
+     *         set_xf_fill() method as its argument.
+     */
     virtual size_t commit_fill() = 0;
 
     // border
 
     virtual void set_border_count(size_t n) = 0;
 
-    virtual void set_border_style(orcus::spreadsheet::border_direction_t dir, border_style_t style) = 0;
+    virtual void set_border_style(border_direction_t dir, border_style_t style) = 0;
     virtual void set_border_color(
-        orcus::spreadsheet::border_direction_t dir, color_elem_t alpha, color_elem_t red, color_elem_t green, color_elem_t blue) = 0;
-    virtual void set_border_width(orcus::spreadsheet::border_direction_t dir, double width, orcus::length_unit_t unit) = 0;
+        border_direction_t dir, color_elem_t alpha, color_elem_t red, color_elem_t green, color_elem_t blue) = 0;
+    virtual void set_border_width(border_direction_t dir, double width, orcus::length_unit_t unit) = 0;
     virtual size_t commit_border() = 0;
 
     // cell protection
@@ -149,8 +251,8 @@ public:
     virtual void set_xf_number_format(size_t index) = 0;
     virtual void set_xf_style_xf(size_t index) = 0;
     virtual void set_xf_apply_alignment(bool b) = 0;
-    virtual void set_xf_horizontal_alignment(orcus::spreadsheet::hor_alignment_t align) = 0;
-    virtual void set_xf_vertical_alignment(orcus::spreadsheet::ver_alignment_t align) = 0;
+    virtual void set_xf_horizontal_alignment(hor_alignment_t align) = 0;
+    virtual void set_xf_vertical_alignment(ver_alignment_t align) = 0;
 
     virtual size_t commit_cell_xf() = 0;
     virtual size_t commit_cell_style_xf() = 0;
@@ -168,21 +270,21 @@ public:
 
 /**
  * Interface for importing sheet properties.  Sheet properties are those
- * that are used for decorative purposes but are not not a part of the
- * sheet cell values.
+ * that are used for decorative purposes but are not necessarily a part of
+ * the sheet cell values.
  */
 class import_sheet_properties
 {
 public:
     ORCUS_DLLPUBLIC virtual ~import_sheet_properties() = 0;
 
-    virtual void set_column_width(orcus::spreadsheet::col_t col, double width, orcus::length_unit_t unit) = 0;
+    virtual void set_column_width(col_t col, double width, orcus::length_unit_t unit) = 0;
 
-    virtual void set_column_hidden(orcus::spreadsheet::col_t col, bool hidden) = 0;
+    virtual void set_column_hidden(col_t col, bool hidden) = 0;
 
-    virtual void set_row_height(orcus::spreadsheet::row_t row, double height, orcus::length_unit_t unit) = 0;
+    virtual void set_row_height(row_t row, double height, orcus::length_unit_t unit) = 0;
 
-    virtual void set_row_hidden(orcus::spreadsheet::row_t row, bool hidden) = 0;
+    virtual void set_row_hidden(row_t row, bool hidden) = 0;
 
     /**
      * Specify merged cell range.
@@ -224,9 +326,9 @@ class import_data_table
 public:
     ORCUS_DLLPUBLIC virtual ~import_data_table() = 0;
 
-    virtual void set_type(orcus::spreadsheet::data_table_type_t type) = 0;
+    virtual void set_type(data_table_type_t type) = 0;
 
-    virtual void set_range(const char* p_range, size_t n_range) = 0;
+    virtual void set_range(const range_t& range) = 0;
 
     virtual void set_first_reference(const char* p_ref, size_t n_ref, bool deleted) = 0;
 
@@ -241,13 +343,12 @@ public:
     ORCUS_DLLPUBLIC virtual ~import_auto_filter() = 0;
 
     /**
-     * Specify the range where the auto filter is applied.  The range
-     * is given in a 2-dimensional A1-style reference.
+     * Specify the range where the auto filter is applied.
      *
-     * @param p_ref pointer to the first character of range string.
-     * @param n_ref length of range string.
+     * @param range structure containing the top-left and bottom-right
+     *              positions of the auto filter range.
      */
-    virtual void set_range(const char* p_ref, size_t n_ref) = 0;
+    virtual void set_range(const range_t& range) = 0;
 
     /**
      * Specify the column position of a filter. The position is relative to
@@ -256,7 +357,7 @@ public:
      * @param col 0-based column position of a filter relative to the first
      *            column.
      */
-    virtual void set_column(orcus::spreadsheet::col_t col) = 0;
+    virtual void set_column(col_t col) = 0;
 
     /**
      * Add a match value to the current column filter.
@@ -320,12 +421,12 @@ public:
      * Sets the type for the formula, value or string of the current condition.
      * Only valid for type = iconset, databar or colorscale.
      */
-    virtual void set_condition_type(orcus::spreadsheet::condition_type_t type) = 0;
+    virtual void set_condition_type(condition_type_t type) = 0;
 
     /**
      * Only valid for type = date.
      */
-    virtual void set_date(orcus::spreadsheet::condition_date_t date) = 0;
+    virtual void set_date(condition_date_t date) = 0;
 
     /**
      * commits the current condition to the current entry.
@@ -348,7 +449,7 @@ public:
      * Position of the 0 axis in the current entry.
      * only valid for type == databar.
      */
-    virtual void set_databar_axis(orcus::spreadsheet::databar_axis_t axis) = 0;
+    virtual void set_databar_axis(databar_axis_t axis) = 0;
 
     /**
      * Databar color for positive values.
@@ -397,16 +498,16 @@ public:
      * Sets the current operation used for the current entry.
      * only valid for type == condition
      */
-    virtual void set_operator(orcus::spreadsheet::condition_operator_t condition_type) = 0;
+    virtual void set_operator(condition_operator_t condition_type) = 0;
 
-    virtual void set_type(orcus::spreadsheet::conditional_format_t type) = 0;
+    virtual void set_type(conditional_format_t type) = 0;
 
     virtual void commit_entry() = 0;
 
     virtual void set_range(const char* p, size_t n) = 0;
 
-    virtual void set_range(orcus::spreadsheet::row_t row_start, orcus::spreadsheet::col_t col_start,
-            orcus::spreadsheet::row_t row_end, orcus::spreadsheet::col_t col_end) = 0;
+    virtual void set_range(row_t row_start, col_t col_start,
+            row_t row_end, col_t col_end) = 0;
 
     virtual void commit_format() = 0;
 };
@@ -437,7 +538,7 @@ public:
     virtual void set_column_identifier(size_t id) = 0;
     virtual void set_column_name(const char* p, size_t n) = 0;
     virtual void set_column_totals_row_label(const char* p, size_t n) = 0;
-    virtual void set_column_totals_row_function(orcus::spreadsheet::totals_row_function_t func) = 0;
+    virtual void set_column_totals_row_function(totals_row_function_t func) = 0;
     virtual void commit_column() = 0;
 
     virtual void set_style_name(const char* p, size_t n) = 0;
@@ -449,6 +550,88 @@ public:
     virtual void commit() = 0;
 };
 
+class ORCUS_DLLPUBLIC import_formula
+{
+public:
+    virtual ~import_formula();
+
+    /**
+     * Set the position of the cell.
+     *
+     * @param row row position.
+     * @param col column position.
+     */
+    virtual void set_position(row_t row, col_t col) = 0;
+
+    /**
+     * Set formula string to the specified cell.
+     *
+     * @param grammar grammar to use to compile the formula string into
+     *                tokens.
+     * @param p pointer to the buffer where the formula string is stored.
+     * @param n size of the buffer where the formula string is stored.
+     */
+    virtual void set_formula(formula_grammar_t grammar, const char* p, size_t n) = 0;
+
+    /**
+     * Register the formula as a shared string, to be shared with other cells.
+     *
+     * @param index shared string index to register the formula with.
+     */
+    virtual void set_shared_formula_index(size_t index) = 0;
+
+    /**
+     * Set cached result of string type.
+     *
+     * @param sindex index of the string value into the shared string pool.
+     */
+    virtual void set_result_string(size_t sindex) = 0;
+
+    /**
+     * Set cached result of numeric type.
+     *
+     * @param value numeric value to set as a cached result.
+     */
+    virtual void set_result_value(double value) = 0;
+
+    /**
+     * Set cached result of boolean type.
+     *
+     * @param value boolean value to set as a cached result.
+     */
+    virtual void set_result_bool(bool value) = 0;
+
+    /**
+     * Set empty value as a cached result.
+     */
+    virtual void set_result_empty() = 0;
+
+    /**
+     * Commit all the formula data to the specified cell.
+     */
+    virtual void commit() = 0;
+};
+
+class ORCUS_DLLPUBLIC import_array_formula
+{
+public:
+    virtual ~import_array_formula();
+
+    virtual void set_range(const range_t& range) = 0;
+
+    virtual void set_formula(formula_grammar_t grammar, const char* p, size_t n) = 0;
+
+    virtual void set_result_string(row_t row, col_t col, size_t sindex) = 0;
+
+    virtual void set_result_value(row_t row, col_t col, double value) = 0;
+
+    virtual void set_result_bool(row_t row, col_t col, bool value) = 0;
+
+    virtual void set_result_empty(row_t row, col_t col) = 0;
+
+    virtual void commit() = 0;
+};
+
 /**
  * Interface for sheet.
  */
@@ -507,6 +690,16 @@ public:
 
     virtual import_named_expression* get_named_expression();
 
+    virtual import_array_formula* get_array_formula();
+
+    /**
+     * Get an interface for importing formula cells.
+     *
+     * @return pointer to the formula interface object, or nullptr if the
+     *         implementer doesn't support importing of formula cells.
+     */
+    virtual import_formula* get_formula();
+
     /**
      * Set raw string value to a cell and have the implementation
      * auto-recognize its data type.
@@ -516,7 +709,7 @@ public:
      * @param p pointer to the first character of the raw string value.
      * @param n size of the raw string value.
      */
-    virtual void set_auto(orcus::spreadsheet::row_t row, orcus::spreadsheet::col_t col, const char* p, size_t n) = 0;
+    virtual void set_auto(row_t row, col_t col, const char* p, size_t n) = 0;
 
     /**
      * Set string value to a cell.
@@ -525,7 +718,7 @@ public:
      * @param col column ID
      * @param sindex 0-based string index in the shared string table.
      */
-    virtual void set_string(orcus::spreadsheet::row_t row, orcus::spreadsheet::col_t col, size_t sindex) = 0;
+    virtual void set_string(row_t row, col_t col, size_t sindex) = 0;
 
     /**
      * Set numerical value to a cell.
@@ -534,7 +727,7 @@ public:
      * @param col column ID
      * @param value value being assigned to the cell.
      */
-    virtual void set_value(orcus::spreadsheet::row_t row, orcus::spreadsheet::col_t col, double value) = 0;
+    virtual void set_value(row_t row, col_t col, double value) = 0;
 
     /**
      * Set a boolean value to a cell.
@@ -543,7 +736,7 @@ public:
      * @param col col ID
      * @param value boolean value being assigned to the cell
      */
-    virtual void set_bool(orcus::spreadsheet::row_t row, orcus::spreadsheet::col_t col, bool value) = 0;
+    virtual void set_bool(row_t row, col_t col, bool value) = 0;
 
     /**
      * Set date and time value to a cell.
@@ -552,7 +745,7 @@ public:
      * @param col column ID
      */
     virtual void set_date_time(
-        orcus::spreadsheet::row_t row, orcus::spreadsheet::col_t col,
+        row_t row, col_t col,
         int year, int month, int day, int hour, int minute, double second) = 0;
 
     /**
@@ -563,7 +756,7 @@ public:
      * @param col column ID
      * @param index 0-based xf (cell format) index
      */
-    virtual void set_format(orcus::spreadsheet::row_t row, orcus::spreadsheet::col_t col, size_t xf_index) = 0;
+    virtual void set_format(row_t row, col_t col, size_t xf_index) = 0;
 
     /**
      * Set cell format to specified cell range.  The cell format is referred
@@ -575,111 +768,19 @@ public:
      * @param col_end end column ID
      * @param index 0-based xf (cell format) index
      */
-    virtual void set_format(orcus::spreadsheet::row_t row_start, orcus::spreadsheet::col_t col_start,
-        orcus::spreadsheet::row_t row_end, orcus::spreadsheet::col_t col_end, size_t xf_index) = 0;
+    virtual void set_format(row_t row_start, col_t col_start,
+        row_t row_end, col_t col_end, size_t xf_index) = 0;
 
     /**
-     * Set normal, non-shared formula expression to specified cell.
+     * Duplicate the value of the source cell to one or more cells located
+     * immediately below it.
      *
-     * @param row row ID
-     * @param col column ID
-     * @param grammar grammar used in the formula expression
-     * @param p pointer to the first character of the raw formula expression
-     *          string.
-     * @param n size of the raw formula expression string.
+     * @param src_row row ID of the source cell
+     * @param src_col column ID of the source cell
+     * @param range_size number of cells below the source cell to copy the
+     *                   source cell value to.  It must be at least one.
      */
-    virtual void set_formula(
-        orcus::spreadsheet::row_t row, orcus::spreadsheet::col_t col, orcus::spreadsheet::formula_grammar_t grammar,
-        const char* p, size_t n) = 0;
-
-    /**
-     * Set shared formula expression to specified cell.  This call also
-     * registers the formula expression to a common formula expression pool to
-     * allow it to be looked up by the specified index later.
-     *
-     * @param row row ID
-     * @param col column ID
-     * @param grammar grammar used in the formula expression
-     * @param sindex shared formula index (0-based)
-     * @param p_formula pointer to the first character of the raw formula
-     *          expression string.
-     * @param n_formula size of the raw formula expression string.
-     * @param p_range pointer to the first character of the range string
-     * @param n_range size of the raw range string
-     */
-    virtual void set_shared_formula(
-        orcus::spreadsheet::row_t row, orcus::spreadsheet::col_t col, orcus::spreadsheet::formula_grammar_t grammar,
-        size_t sindex, const char* p_formula, size_t n_formula, const char* p_range, size_t n_range) = 0;
-
-    /**
-     * Set shared formula expression to specified cell.  This call also
-     * registers the formula expression to a common formula expression pool to
-     * allow it to be looked up by the specified index later.
-     *
-     * @param row row ID
-     * @param col column ID
-     * @param grammar grammar used in the formula expression
-     * @param sindex shared formula index (0-based)
-     * @param p_formula pointer to the first character of the raw formula
-     *          expression string.
-     * @param n_formula size of the raw formula expression string.
-     */
-    virtual void set_shared_formula(
-        orcus::spreadsheet::row_t row, orcus::spreadsheet::col_t col, orcus::spreadsheet::formula_grammar_t grammar,
-        size_t sindex, const char* p_formula, size_t n_formula) = 0;
-
-    /**
-     * Set shared formula to specified cell by shared formula index.  The
-     * formula expression itself associated with the index must be defined.
-     *
-     * @param row row ID
-     * @param col column ID
-     * @param grammar grammar used in the formula expression
-     * @param sindex shared formula index (0-based)
-     * @param p pointer to the first character of the raw formula expression
-     *          string.
-     * @param n size of the raw formula expression string.
-     */
-    virtual void set_shared_formula(
-        orcus::spreadsheet::row_t row, orcus::spreadsheet::col_t col, size_t sindex) = 0;
-
-    virtual void set_formula_result(
-        orcus::spreadsheet::row_t row, orcus::spreadsheet::col_t col, double value) = 0;
-
-    virtual void set_formula_result(
-        orcus::spreadsheet::row_t row, orcus::spreadsheet::col_t col, const char* p, size_t n) = 0;
-
-    /**
-     * Set a array formula to the specified cells. The formula covers an area
-     * specified by array_rows and array_cols beginning from the base cell.
-     *
-     * @param row row ID
-     * @param col column ID
-     * @param grammar grammar used in the formula expression
-     * @param p pointer to the first character of the raw formula expression string.
-     * @param n size of the raw formula expression string
-     * @param array_rows number of rows the array formula covers
-     * @param array_cols number of columns the array formula covers
-     */
-    virtual void set_array_formula(
-        orcus::spreadsheet::row_t row, orcus::spreadsheet::col_t col, orcus::spreadsheet::formula_grammar_t grammar,
-        const char* p, size_t n, orcus::spreadsheet::row_t array_rows, orcus::spreadsheet::col_t array_cols) = 0;
-
-    /**
-     * Set a array formula to the specified cells. The formula covers an area
-     * specified by the range string.
-     *
-     * @param row row ID
-     * @param col column ID
-     * @param grammar grammar used in the formula expression
-     * @param p pointer to the first character of the raw formula expression string.
-     * @param n size of the raw formula expression string
-     * @param p_range pointer to the first character of the range string
-     * @param n_range size of the range string
-     */
-    virtual void set_array_formula(
-        orcus::spreadsheet::row_t row, orcus::spreadsheet::col_t col, orcus::spreadsheet::formula_grammar_t grammar,
-        const char* p, size_t n, const char* p_range, size_t n_range) = 0;
+    virtual void fill_down_cells(row_t src_row, col_t src_col, row_t range_size) = 0;
 
     /**
      * Get the size of the sheet.
@@ -687,7 +788,7 @@ public:
      * @return structure containing the numbers of rows and columns of the
      *         sheet.
      */
-    virtual orcus::spreadsheet::range_size_t get_sheet_size() const = 0;
+    virtual range_size_t get_sheet_size() const = 0;
 };
 
 class import_global_settings
@@ -714,14 +815,21 @@ public:
      *
      * @param grammar default formula grammar
      */
-    virtual void set_default_formula_grammar(orcus::spreadsheet::formula_grammar_t grammar) = 0;
+    virtual void set_default_formula_grammar(formula_grammar_t grammar) = 0;
 
     /**
      * Get current default formula grammar.
      *
      * @return current default formula grammar.
      */
-    virtual orcus::spreadsheet::formula_grammar_t get_default_formula_grammar() const = 0;
+    virtual formula_grammar_t get_default_formula_grammar() const = 0;
+
+    /**
+     * Set the character set to be used when parsing string values.
+     *
+     * @param charset character set to apply when parsing string values.
+     */
+    virtual void set_character_set(character_set_t charset) = 0;
 };
 
 class ORCUS_DLLPUBLIC import_reference_resolver
@@ -805,7 +913,7 @@ public:
      *         NULL if the client app doesn't support pivot tables.
      */
     virtual import_pivot_cache_definition* create_pivot_cache_definition(
-        orcus::spreadsheet::pivot_cache_id_t cache_id);
+        pivot_cache_id_t cache_id);
 
     /**
      * Create an interface for pivot cache records import for a specified
@@ -817,7 +925,7 @@ public:
      *         return nullptr if the client app doesn't support pivot tables.
      */
     virtual import_pivot_cache_records* create_pivot_cache_records(
-        orcus::spreadsheet::pivot_cache_id_t cache_id);
+        pivot_cache_id_t cache_id);
 
     /**
      * Append a sheet with specified sheet position index and name.
@@ -833,11 +941,11 @@ public:
      *         client app fails to append a new sheet.
      */
     virtual import_sheet* append_sheet(
-        orcus::spreadsheet::sheet_t sheet_index, const char* sheet_name, size_t sheet_name_length) = 0;
+        sheet_t sheet_index, const char* sheet_name, size_t sheet_name_length) = 0;
 
     /**
      * @return pointer to the sheet instance whose name matches the name
-     *         passed to this method. It returns NULL if no sheet instance
+     *         passed to this method. It returns nullptr if no sheet instance
      *         exists by the specified name.
      */
     virtual import_sheet* get_sheet(const char* sheet_name, size_t sheet_name_length) = 0;
@@ -847,10 +955,10 @@ public:
      *
      * @param sheet_index sheet index
      *
-     * @return pointer to the sheet instance, or NULL if no sheet instance
+     * @return pointer to the sheet instance, or nullptr if no sheet instance
      *         exists at specified sheet index position.
      */
-    virtual import_sheet* get_sheet(orcus::spreadsheet::sheet_t sheet_index) = 0;
+    virtual import_sheet* get_sheet(sheet_t sheet_index) = 0;
 
     /**
      * This method is called at the end of import, to give the implementor a
