@@ -32,8 +32,8 @@ struct header_cell
     spreadsheet::col_t col;
     pstring value;
 
-    header_cell(spreadsheet::row_t row, spreadsheet::col_t col, const pstring& value) :
-        row(row), col(col), value(value) {}
+    header_cell(spreadsheet::row_t _row, spreadsheet::col_t _col, const pstring& _value) :
+        row(_row), col(_col), value(_value) {}
 };
 
 class max_row_size_reached {};
@@ -136,8 +136,8 @@ orcus_csv::orcus_csv(spreadsheet::iface::import_factory* factory) :
 
 void orcus_csv::read_file(const string& filepath)
 {
-    string strm = load_file_content(filepath.c_str());
-    parse(strm.data(), strm.size());
+    file_content fc(filepath.data());
+    parse(fc.data(), fc.size());
 
     mp_factory->finalize();
 }
