@@ -23,34 +23,87 @@ namespace {
 
 class mock_ref_resolver : public import_reference_resolver
 {
+    virtual address_t resolve_address(const char* p, size_t n) override
+    {
+        address_t ret;
+        ret.row = 0;
+        ret.column = 0;
+
+        return ret;
+    }
+
+    virtual range_t resolve_range(const char* p, size_t n) override
+    {
+        range_t ret;
+        ret.first.row = 0;
+        ret.first.column = 0;
+        ret.last.row = 0;
+        ret.last.column = 0;
+
+        return ret;
+    }
+};
+
+class mock_array_formula : public import_array_formula
+{
+public:
+    virtual void set_range(const range_t& range) override
+    {
+        assert(range.first.row == 2);
+        assert(range.first.column == 1);
+        assert(range.last.row == 3);
+        assert(range.last.column == 1);
+    }
+
+    virtual void set_formula(formula_grammar_t grammar, const char* p, size_t n) override
+    {
+        assert(grammar == formula_grammar_t::xlsx);
+        assert(string(p, n) == "A1:A2");
+    }
+
+    virtual void set_result_bool(row_t row, col_t col, bool value) override
+    {
+    }
+
+    virtual void set_result_empty(row_t row, col_t col) override
+    {
+    }
+
+    virtual void set_result_string(row_t row, col_t col, size_t sindex) override
+    {
+    }
+
+    virtual void set_result_value(row_t row, col_t col, double value) override
+    {
+    }
+
+    virtual void commit() override
+    {
+    }
 };
 
 class mock_sheet : public import_sheet
 {
-public:
+    mock_array_formula m_array_formula;
 
-    virtual void set_value(row_t row, col_t col, double val)
+public:
+    virtual void set_value(row_t row, col_t col, double val) override
     {
         assert(row == -1);
         assert(col == 0);
         assert(val == 5.0);
     }
 
-    virtual void set_bool(row_t row, col_t col, bool val)
+    virtual void set_bool(row_t row, col_t col, bool val) override
     {
         assert(row == -1);
         assert(col == 0);
         assert(val == true);
     }
 
-    virtual void set_array_formula(row_t row, col_t col, formula_grammar_t grammar,
-            const char* s, size_t n, const char* s_range, size_t n_range)
+    virtual iface::import_array_formula* get_array_formula() override
     {
-        assert(row == -1);
-        assert(col == 0);
-        assert(grammar == formula_grammar_t::xlsx_2007);
-        assert(string(s, n) == "A1:A2");
-        assert(string(s_range, n_range) == "B3:B4");
+        return &m_array_formula;
     }
 };
 
