@@ -26,6 +26,7 @@ xlsx_workbook_context::xlsx_workbook_context(
     spreadsheet::iface::import_factory& factory) :
     xml_context_base(session_cxt, tokens),
     m_defined_name_scope(-1),
+    m_sheet_count(0),
     m_factory(factory),
     mp_named_exp(factory.get_named_expression()) {}
 
@@ -50,7 +51,6 @@ void xlsx_workbook_context::start_element(xmlns_id_t ns, xml_token_t name, const
     xml_token_pair_t parent = push_stack(ns, name);
     session_context& cxt = get_session_context();
     string_pool& sp = cxt.m_string_pool;
-    xlsx_session_data& sdata = static_cast<xlsx_session_data&>(*cxt.mp_data);
 
     switch (name)
     {
@@ -100,15 +100,9 @@ void xlsx_workbook_context::start_element(xmlns_id_t ns, xml_token_t name, const
                 }
             );
 
-            if (sheet.id > 0)
-            {
-                // Excel's sheet ID is 1-based. Convert it to 0-based.
-                sdata.set_sheet_name_map(sheet.name, sheet.id-1);
-
-                // Insert the sheet here so that we have all the sheets available
-                // prior to parsing global named expressions.
-                m_factory.append_sheet(sheet.id-1, sheet.name.data(), sheet.name.size());
-            }
+            // Insert the sheet here so that we have all the sheets available
+            // prior to parsing global named expressions.
+            m_factory.append_sheet(m_sheet_count++, sheet.name.data(), sheet.name.size());
 
             m_workbook_info.data.insert(
                 opc_rel_extras_t::map_type::value_type(
