@@ -16,6 +16,7 @@
 
 #include <cassert>
 #include <algorithm>
+#include <iostream>
 
 using namespace std;
 
@@ -23,19 +24,6 @@ namespace orcus {
 
 namespace {
 
-template<typename T>
-class find_by_name : std::unary_function<std::unique_ptr<T>, bool>
-{
-    xmlns_id_t m_ns;
-    pstring m_name;
-public:
-    find_by_name(xmlns_id_t ns, const pstring& name) : m_ns(ns), m_name(name) {}
-    bool operator() (const std::unique_ptr<T>& e) const
-    {
-        return m_ns == e->ns && m_name == e->name;
-    }
-};
-
 class xpath_parser
 {
     const xmlns_context& m_cxt;
@@ -147,109 +135,48 @@ void print_element_stack(ostream& os, const T& elem_stack)
 
 xml_map_tree::xpath_error::xpath_error(const string& msg) : general_error(msg) {}
 
-xml_map_tree::cell_position::cell_position() :
-    row(-1), col(-1) {}
-
-xml_map_tree::cell_position::cell_position(const pstring& _sheet, spreadsheet::row_t _row, spreadsheet::col_t _col) :
-    sheet(_sheet), row(_row), col(_col) {}
-
-xml_map_tree::cell_position::cell_position(const cell_position& r) :
-    sheet(r.sheet), row(r.row), col(r.col) {}
-
 xml_map_tree::element_position::element_position() :
     open_begin(0), open_end(0), close_begin(0), close_end(0) {}
 
 xml_map_tree::cell_reference::cell_reference() {}
 
 xml_map_tree::range_reference::range_reference(const cell_position& _pos) :
-    pos(_pos), row_size(0) {}
+    pos(_pos), row_position(0) {}
 
-xml_map_tree::linkable::linkable(xmlns_id_t _ns, const pstring& _name, linkable_node_type _node_type) :
-    ns(_ns), name(_name), node_type(_node_type) {}
-
-xml_map_tree::attribute::attribute(xmlns_id_t _ns, const pstring& _name, reference_type _ref_type) :
-    linkable(_ns, _name, node_attribute), ref_type(_ref_type)
+xml_map_tree::linkable::linkable(
+    xml_map_tree& parent, xmlns_id_t _ns, const pstring& _name, linkable_node_type _node_type, reference_type _ref_type) :
+    ns(_ns), name(_name), node_type(_node_type), ref_type(_ref_type)
 {
-    switch (ref_type)
-    {
-        case reference_cell:
-            cell_ref = new cell_reference;
-        break;
-        case reference_range_field:
-            field_ref = new field_in_range;
-        break;
-        default:
-            throw general_error("unexpected reference type in the constructor of attribute.");
-    }
+    parent.create_ref_store(*this);
 }
 
-xml_map_tree::attribute::~attribute()
-{
-    switch (ref_type)
-    {
-        case reference_cell:
-            delete cell_ref;
-        break;
-        case reference_range_field:
-            delete field_ref;
-        break;
-        default:
-            assert(!"unexpected reference type in the destructor of attribute.");
-    }
-}
+xml_map_tree::attribute::attribute(args_type args) :
+    linkable(std::get<0>(args), std::get<1>(args), std::get<2>(args), node_attribute, std::get<3>(args)) {}
 
-xml_map_tree::element::element(
-    xmlns_id_t _ns, const pstring& _name, element_type _elem_type, reference_type _ref_type) :
-    linkable(_ns, _name, node_element),
-    elem_type(_elem_type),
-    ref_type(_ref_type),
-    range_parent(nullptr)
+xml_map_tree::attribute::~attribute() {}
+
+xml_map_tree::element::element(args_type args) :
+    linkable(std::get<0>(args), std::get<1>(args), std::get<2>(args), node_element, std::get<4>(args)),
+    elem_type(std::get<3>(args)),
+    child_elements(nullptr),
+    range_parent(nullptr),
+    row_group(nullptr),
+    row_group_position(0)
 {
+    xml_map_tree& parent = std::get<0>(args);
+
     if (elem_type == element_unlinked)
     {
-        child_elements = new element_store_type;
+        child_elements = parent.m_element_store_pool.construct();
         return;
     }
 
     assert(elem_type == element_linked);
-
-    switch (ref_type)
-    {
-        case reference_cell:
-            cell_ref = new cell_reference;
-        break;
-        case reference_range_field:
-            field_ref = new field_in_range;
-        break;
-        default:
-            throw general_error("unexpected reference type in the constructor of element.");
-    }
 }
 
-xml_map_tree::element::~element()
-{
-    if (elem_type == element_unlinked)
-    {
-        delete child_elements;
-        return;
-    }
+xml_map_tree::element::~element() {}
 
-    assert(elem_type == element_linked);
-
-    switch (ref_type)
-    {
-        case reference_cell:
-            delete cell_ref;
-        break;
-        case reference_range_field:
-            delete field_ref;
-        break;
-        default:
-            assert(!"unexpected reference type in the destructor of element.");
-    }
-}
-
-const xml_map_tree::element* xml_map_tree::element::get_child(xmlns_id_t _ns, const pstring& _name) const
+xml_map_tree::element* xml_map_tree::element::get_child(xmlns_id_t _ns, const pstring& _name)
 {
     if (elem_type != element_unlinked)
         return nullptr;
@@ -257,9 +184,98 @@ const xml_map_tree::element* xml_map_tree::element::get_child(xmlns_id_t _ns, co
     assert(child_elements);
 
     auto it = std::find_if(
-        child_elements->begin(), child_elements->end(), find_by_name<element>(_ns, _name));
+        child_elements->begin(), child_elements->end(),
+        [&_ns,&_name](const element* p) -> bool
+        {
+            return p->ns == _ns && p->name == _name;
+        }
+    );
 
-    return it == child_elements->end() ? nullptr : it->get();
+    return it == child_elements->end() ? nullptr : *it;
+}
+
+xml_map_tree::element* xml_map_tree::element::get_or_create_child(
+    xml_map_tree& parent, xmlns_id_t _ns, const pstring& _name)
+{
+    auto it = std::find_if(
+        child_elements->begin(), child_elements->end(),
+        [&_ns,&_name](const element* p) -> bool
+        {
+            return p->ns == _ns && p->name == _name;
+        }
+    );
+
+    if (it != child_elements->end())
+        return *it;
+
+    string_pool& sp = parent.m_names;
+
+    // Insert a new element of this name.
+    auto const nm = sp.intern(_name.get(), _name.size()).first; // work around LLVM < 7 libc++ bug
+    child_elements->push_back(
+        parent.m_element_pool.construct(
+            element::args_type(
+                parent,
+                _ns,
+                nm,
+                element_unlinked,
+                reference_unknown
+            )
+        )
+    );
+
+    return child_elements->back();
+}
+
+xml_map_tree::element* xml_map_tree::element::get_or_create_linked_child(
+    xml_map_tree& parent, xmlns_id_t _ns, const pstring& _name, reference_type _ref_type)
+{
+    auto it = std::find_if(
+        child_elements->begin(), child_elements->end(),
+        [&_ns,&_name](const element* p) -> bool
+        {
+            return p->ns == _ns && p->name == _name;
+        }
+    );
+
+    if (it != child_elements->end())
+    {
+        // Specified child element already exists. Make sure it's unlinked.
+        element* elem = *it;
+        if (elem->ref_type != reference_unknown || elem->elem_type != element_unlinked)
+            throw xpath_error("This element is already linked.  You can't link the same element twice.");
+
+        elem->link_reference(parent, _ref_type);
+        return elem;
+    }
+
+    string_pool& sp = parent.m_names;
+
+    // Insert a new linked element of this name.
+    auto const nm = sp.intern(_name.get(), _name.size()).first; // work around LLVM < 7 libc++ bug
+    child_elements->push_back(
+        parent.m_element_pool.construct(
+            element::args_type(
+                parent,
+                _ns,
+                nm,
+                element_linked,
+                _ref_type
+            )
+        )
+    );
+
+    return child_elements->back();
+}
+
+void xml_map_tree::element::link_reference(xml_map_tree& parent, reference_type _ref_type)
+{
+    if (elem_type == element_unlinked)
+        parent.m_element_store_pool.destroy(child_elements);
+
+    elem_type = element_linked;
+    ref_type = _ref_type;
+    parent.create_ref_store(*this);
 }
 
 bool xml_map_tree::element::unlinked_attribute_anchor() const
@@ -278,7 +294,7 @@ void xml_map_tree::walker::reset()
     m_unlinked_stack.clear();
 }
 
-const xml_map_tree::element* xml_map_tree::walker::push_element(xmlns_id_t ns, const pstring& name)
+xml_map_tree::element* xml_map_tree::walker::push_element(xmlns_id_t ns, const pstring& name)
 {
     if (!m_unlinked_stack.empty())
     {
@@ -296,7 +312,7 @@ const xml_map_tree::element* xml_map_tree::walker::push_element(xmlns_id_t ns, c
             return nullptr;
         }
 
-        const element* p = m_parent.mp_root;
+        element* p = m_parent.mp_root;
         if (p->ns != ns || p->name != name)
         {
             // Names differ.
@@ -311,7 +327,7 @@ const xml_map_tree::element* xml_map_tree::walker::push_element(xmlns_id_t ns, c
     if (m_stack.back()->elem_type == element_unlinked)
     {
         // Check if the current element has a child of the same name.
-        const element* p = m_stack.back()->get_child(ns, name);
+        element* p = m_stack.back()->get_child(ns, name);
         if (p)
         {
             m_stack.push_back(p);
@@ -323,7 +339,7 @@ const xml_map_tree::element* xml_map_tree::walker::push_element(xmlns_id_t ns, c
     return nullptr;
 }
 
-const xml_map_tree::element* xml_map_tree::walker::pop_element(xmlns_id_t ns, const pstring& name)
+xml_map_tree::element* xml_map_tree::walker::pop_element(xmlns_id_t ns, const pstring& name)
 {
     if (!m_unlinked_stack.empty())
     {
@@ -351,13 +367,9 @@ const xml_map_tree::element* xml_map_tree::walker::pop_element(xmlns_id_t ns, co
 }
 
 xml_map_tree::xml_map_tree(xmlns_repository& xmlns_repo) :
-    m_xmlns_cxt(xmlns_repo.create_context()), mp_cur_range_ref(nullptr), mp_root(nullptr) {}
+    m_xmlns_cxt(xmlns_repo.create_context()), mp_root(nullptr) {}
 
-xml_map_tree::~xml_map_tree()
-{
-    std::for_each(m_field_refs.begin(), m_field_refs.end(), map_object_deleter<range_ref_map_type>());
-    delete mp_root;
-}
+xml_map_tree::~xml_map_tree() {}
 
 void xml_map_tree::set_namespace_alias(const pstring& alias, const pstring& uri)
 {
@@ -384,21 +396,20 @@ void xml_map_tree::set_cell_link(const pstring& xpath, const cell_position& ref)
     cout << "xml_map_tree::set_cell_link: xpath='" << xpath << "' (ref=" << ref << ")" << endl;
 #endif
 
-    element_list_type elem_stack;
-    linkable* node = get_element_stack(xpath, reference_cell, elem_stack);
-    assert(node);
-    assert(!elem_stack.empty());
+    linked_node_type linked_node = get_linked_node(xpath, reference_cell);
+    assert(linked_node.node);
+    assert(!linked_node.elem_stack.empty());
     cell_reference* cell_ref = nullptr;
-    switch (node->node_type)
+    switch (linked_node.node->node_type)
     {
         case node_element:
-            assert(static_cast<element*>(node)->cell_ref);
-            cell_ref = static_cast<element*>(node)->cell_ref;
-        break;
+            assert(static_cast<element*>(linked_node.node)->cell_ref);
+            cell_ref = static_cast<element*>(linked_node.node)->cell_ref;
+            break;
         case node_attribute:
-            assert(static_cast<attribute*>(node)->cell_ref);
-            cell_ref = static_cast<attribute*>(node)->cell_ref;
-        break;
+            assert(static_cast<attribute*>(linked_node.node)->cell_ref);
+            cell_ref = static_cast<attribute*>(linked_node.node)->cell_ref;
+            break;
         default:
             throw general_error("unknown node type returned from get_element_stack call in xml_map_tree::set_cell_link().");
     }
@@ -406,64 +417,53 @@ void xml_map_tree::set_cell_link(const pstring& xpath, const cell_position& ref)
     cell_ref->pos = ref;
 }
 
-void xml_map_tree::start_range()
+void xml_map_tree::start_range(const cell_position& pos)
 {
-    m_cur_range_parent.clear();
-    mp_cur_range_ref = nullptr;
+    m_cur_range_field_links.clear();
+    m_cur_range_pos = pos;
 }
 
-void xml_map_tree::append_range_field_link(const pstring& xpath, const cell_position& pos)
+void xml_map_tree::append_range_field_link(const pstring& xpath)
 {
     if (xpath.empty())
         return;
 
-    range_reference* range_ref = nullptr;
-    range_ref_map_type::iterator it = m_field_refs.lower_bound(pos);
-    if (it == m_field_refs.end() || m_field_refs.key_comp()(pos, it->first))
-    {
-        // This reference does not exist yet.  Insert a new one.
+    m_cur_range_field_links.emplace_back(xpath);
+}
 
-        // Make sure the sheet name string is persistent.
-        cell_position pos_safe = pos;
-        pos_safe.sheet = m_names.intern(pos.sheet.get(), pos.sheet.size()).first;
-
-        it = m_field_refs.insert(it, range_ref_map_type::value_type(pos_safe, new range_reference(pos_safe)));
-    }
-
-    range_ref = it->second;
-    assert(range_ref);
-
-    if (!mp_cur_range_ref)
-        mp_cur_range_ref = range_ref;
-
-#if ORCUS_DEBUG_XML_MAP_TREE
-    cout << "xml_map_tree::append_range_field_link: " << xpath << " (ref=" << pos << ")" << endl;
-#endif
-    element_list_type elem_stack;
-    linkable* node = get_element_stack(xpath, reference_range_field, elem_stack);
-    if (elem_stack.size() < 2)
+void xml_map_tree::insert_range_field_link(
+    range_reference& range_ref, element_list_type& range_parent, const pstring& xpath)
+{
+    linked_node_type linked_node = get_linked_node(xpath, reference_range_field);
+    if (linked_node.elem_stack.size() < 2)
         throw xpath_error("Path of a range field link must be at least 2 levels.");
 
-    switch (node->node_type)
+    if (linked_node.node->node_type == node_unknown)
+        throw xpath_error("Unrecognized node type");
+
+    if (linked_node.anchor_elem)
+        linked_node.anchor_elem->linked_range_fields.push_back(range_ref.field_nodes.size());
+
+    switch (linked_node.node->node_type)
     {
         case node_element:
         {
-            element* p = static_cast<element*>(node);
+            element* p = static_cast<element*>(linked_node.node);
             assert(p && p->ref_type == reference_range_field && p->field_ref);
-            p->field_ref->ref = range_ref;
-            p->field_ref->column_pos = range_ref->field_nodes.size();
+            p->field_ref->ref = &range_ref;
+            p->field_ref->column_pos = range_ref.field_nodes.size();
 
-            range_ref->field_nodes.push_back(p);
+            range_ref.field_nodes.push_back(p);
         }
         break;
         case node_attribute:
         {
-            attribute* p = static_cast<attribute*>(node);
+            attribute* p = static_cast<attribute*>(linked_node.node);
             assert(p && p->ref_type == reference_range_field && p->field_ref);
-            p->field_ref->ref = range_ref;
-            p->field_ref->column_pos = range_ref->field_nodes.size();
+            p->field_ref->ref = &range_ref;
+            p->field_ref->column_pos = range_ref.field_nodes.size();
 
-            range_ref->field_nodes.push_back(p);
+            range_ref.field_nodes.push_back(p);
         }
         break;
         default:
@@ -472,63 +472,89 @@ void xml_map_tree::append_range_field_link(const pstring& xpath, const cell_posi
 
     // Determine the deepest common element for all field link elements in the
     // current range reference.
-    if (m_cur_range_parent.empty())
+    if (range_parent.empty())
     {
         // First field link in this range.
-        element_list_type::iterator it_end = elem_stack.end();
-        if (node->node_type == node_element)
+        element_list_type::iterator it_end = linked_node.elem_stack.end();
+        if (linked_node.node->node_type == node_element)
             --it_end; // Skip the linked element, which is used as a field in a range.
 
         --it_end; // Skip the next-up element, which is used to group a single record entry.
-        m_cur_range_parent.assign(elem_stack.begin(), it_end);
+        range_parent.assign(linked_node.elem_stack.begin(), it_end);
 #if ORCUS_DEBUG_XML_MAP_TREE
-        print_element_stack(cout, m_cur_range_parent);
+        print_element_stack(cout, range_parent);
         cout << endl;
 #endif
     }
     else
     {
         // Determine the deepest common element between the two.
-        element_list_type::iterator it = elem_stack.begin(), it_end = elem_stack.end();
-        element_list_type::iterator it_cur = m_cur_range_parent.begin(), it_cur_end = m_cur_range_parent.end();
-        if (*it != *it_cur)
+        element_list_type::iterator it_elem = linked_node.elem_stack.begin(), it_elem_end = linked_node.elem_stack.end();
+        element_list_type::iterator it_cur = range_parent.begin(), it_cur_end = range_parent.end();
+        if (*it_elem != *it_cur)
             throw xpath_error("Two field links in the same range reference start with different root elements.");
 
-        ++it;
+        ++it_elem;
         ++it_cur;
 
-        for (; it != it_end && it_cur != it_cur_end; ++it, ++it_cur)
+        for (; it_elem != it_elem_end && it_cur != it_cur_end; ++it_elem, ++it_cur)
         {
-            if (*it == *it_cur)
+            if (*it_elem == *it_cur)
                 continue;
 
             // The two elements differ.  Take their parent element as the new common element.
-            m_cur_range_parent.assign(elem_stack.begin(), it); // current elemnt excluded.
+            range_parent.assign(linked_node.elem_stack.begin(), it_elem); // current elemnt excluded.
             break;
         }
 
-        if (m_cur_range_parent.empty())
+        if (range_parent.empty())
             throw xpath_error("Two field links in the same range reference must at least share the first level of their paths.");
     }
 }
 
+void xml_map_tree::set_range_row_group(const pstring& xpath)
+{
+    if (xpath.empty())
+        return;
+
+    range_reference* range_ref = get_range_reference(m_cur_range_pos);
+    assert(range_ref);
+
+    element* elem = get_element(xpath);
+    assert(elem);
+    elem->row_group = range_ref;
+}
+
 void xml_map_tree::commit_range()
 {
-    if (!mp_cur_range_ref)
+    if (m_cur_range_field_links.empty())
         // Nothing to commit.
         return;
 
+    range_reference* range_ref = get_range_reference(m_cur_range_pos);
+    assert(range_ref);
+
+    // commont parent element for this range.
+    element_list_type range_parent;
+
+    for (const pstring& path : m_cur_range_field_links)
+        insert_range_field_link(*range_ref, range_parent, path);
+
 #if ORCUS_DEBUG_XML_MAP_TREE
     cout << "parent element path for this range: ";
-    element_list_type::iterator it = m_cur_range_parent.begin(), it_end = m_cur_range_parent.end();
+    element_list_type::iterator it = range_parent.begin(), it_end = range_parent.end();
     for (; it != it_end; ++it)
         cout << "/" << (**it).name;
     cout << endl;
 #endif
 
-    assert(!m_cur_range_parent.empty());
+    assert(!range_parent.empty());
     // Mark the range parent element.
-    m_cur_range_parent.back()->range_parent = mp_cur_range_ref;
+    range_parent.back()->range_parent = range_ref;
+
+    // Set the current position invalid.
+    m_cur_range_pos.row = -1;
+    m_cur_range_pos.col = -1;
 }
 
 const xml_map_tree::linkable* xml_map_tree::get_link(const pstring& xpath) const
@@ -566,13 +592,18 @@ const xml_map_tree::linkable* xml_map_tree::get_link(const pstring& xpath) const
             const element* elem = static_cast<const element*>(cur_node);
             const attribute_store_type& attrs = elem->attributes;
             auto it = std::find_if(
-                attrs.begin(), attrs.end(), find_by_name<attribute>(token.ns, token.name));
+                attrs.begin(), attrs.end(),
+                [&token](const attribute* p) -> bool
+                {
+                    return p->ns == token.ns && p->name == token.name;
+                }
+            );
 
             if (it == attrs.end())
                 // No such attribute exists.
                 return nullptr;
 
-            return it->get();
+            return *it;
         }
 
         // See if an element of this name exists below the current element.
@@ -589,13 +620,17 @@ const xml_map_tree::linkable* xml_map_tree::get_link(const pstring& xpath) const
 
         auto it = std::find_if(
             elem->child_elements->begin(), elem->child_elements->end(),
-            find_by_name<element>(token.ns, token.name));
+            [&token](const element* p) -> bool
+            {
+                return p->ns == token.ns && p->name == token.name;
+            }
+        );
 
         if (it == elem->child_elements->end())
             // No such child element exists.
             return nullptr;
 
-        cur_node = it->get();
+        cur_node = *it;
     }
 
     if (cur_node->node_type != node_element || static_cast<const element*>(cur_node)->elem_type == element_unlinked)
@@ -620,13 +655,47 @@ pstring xml_map_tree::intern_string(const pstring& str) const
     return m_names.intern(str).first;
 }
 
-xml_map_tree::linkable* xml_map_tree::get_element_stack(
-    const pstring& xpath, reference_type ref_type, element_list_type& elem_stack)
+xml_map_tree::range_reference* xml_map_tree::get_range_reference(const cell_position& pos)
 {
-    assert(!xpath.empty());
-    xpath_parser parser(m_xmlns_cxt,xpath.get(), xpath.size());
+    range_ref_map_type::iterator it = m_field_refs.lower_bound(pos);
+    if (it == m_field_refs.end() || m_field_refs.key_comp()(pos, it->first))
+    {
+        // This reference does not exist yet.  Insert a new one.
 
-    element_list_type elem_stack_new;
+        // Make sure the sheet name string is persistent.
+        cell_position pos_safe = pos;
+        pos_safe.sheet = m_names.intern(pos.sheet.get(), pos.sheet.size()).first;
+
+        it = m_field_refs.insert(
+            it, range_ref_map_type::value_type(
+                pos_safe,
+                m_range_reference_pool.construct(pos_safe)));
+    }
+
+    return it->second;
+}
+
+void xml_map_tree::create_ref_store(linkable& node)
+{
+    switch (node.ref_type)
+    {
+        case xml_map_tree::reference_cell:
+            node.cell_ref = m_cell_reference_pool.construct();
+            break;
+        case xml_map_tree::reference_range_field:
+            node.field_ref = m_field_in_range_pool.construct();
+            break;
+        case xml_map_tree::reference_unknown:
+            break;
+    }
+}
+
+xml_map_tree::linked_node_type xml_map_tree::get_linked_node(const pstring& xpath, reference_type ref_type)
+{
+    linked_node_type ret;
+
+    assert(!xpath.empty());
+    xpath_parser parser(m_xmlns_cxt, xpath.get(), xpath.size());
 
     // Get the root element first.
     xpath_parser::token token = parser.next();
@@ -642,16 +711,25 @@ xml_map_tree::linkable* xml_map_tree::get_element_stack(
         if (token.attribute)
             throw xpath_error("root element cannot be an attribute.");
 
-        mp_root = new element(
-            token.ns, m_names.intern(token.name.get(), token.name.size()).first,
-            element_unlinked, reference_unknown);
+        auto const nm = m_names.intern(token.name).first; // work around LLVM < 7 libc++ bug
+        mp_root = m_element_pool.construct(
+            element::args_type(
+                *this,
+                token.ns,
+                nm,
+                element_unlinked,
+                reference_unknown
+            )
+        );
     }
 
-    elem_stack_new.push_back(mp_root);
-    element* cur_element = elem_stack_new.back();
+    ret.elem_stack.push_back(mp_root);
+    element* cur_element = ret.elem_stack.back();
     assert(cur_element);
     assert(cur_element->child_elements);
 
+    element* row_group_elem = nullptr;
+
     token = parser.next();
     for (xpath_parser::token token_next = parser.next(); !token_next.name.empty(); token_next = parser.next())
     {
@@ -659,30 +737,18 @@ xml_map_tree::linkable* xml_map_tree::get_element_stack(
         if (token.attribute)
             throw xpath_error("attribute must always be at the end of the path.");
 
-        element_store_type& children = *cur_element->child_elements;
-        auto it = std::find_if(
-            children.begin(), children.end(), find_by_name<element>(token.ns, token.name));
-        if (it == children.end())
-        {
-            // Insert a new element of this name.
-            children.push_back(
-                orcus::make_unique<element>(
-                    token.ns, m_names.intern(token.name.get(), token.name.size()).first,
-                    element_unlinked, reference_unknown));
-            cur_element = children.back().get();
-        }
-        else
-            cur_element = it->get();
-
-        elem_stack_new.push_back(cur_element);
+        cur_element = cur_element->get_or_create_child(*this, token.ns, token.name);
+        ret.elem_stack.push_back(cur_element);
         token = token_next;
+
+        if (cur_element->row_group)
+            row_group_elem = cur_element;
     }
 
     assert(cur_element);
 
     // Insert a leaf node.
 
-    linkable* ret = nullptr;
     if (token.attribute)
     {
         // This is an attribute.  Insert it into the current element.
@@ -690,70 +756,88 @@ xml_map_tree::linkable* xml_map_tree::get_element_stack(
 
         // Check if an attribute of the same name already exists.
         auto it = std::find_if(
-            attrs.begin(), attrs.end(), find_by_name<attribute>(token.ns, token.name));
+            attrs.begin(), attrs.end(),
+            [&token](const attribute* p) -> bool
+            {
+                return p->ns == token.ns && p->name == token.name;
+            }
+        );
+
         if (it != attrs.end())
             throw xpath_error("This attribute is already linked.  You can't link the same attribute twice.");
 
-        attrs.push_back(
-            orcus::make_unique<attribute>(
-                token.ns, m_names.intern(token.name.get(), token.name.size()).first, ref_type));
+        auto const nm = m_names.intern(token.name.get(), token.name.size()).first; // work around LLVM < 7 libc++ bug
+        attribute* p = m_attribute_pool.construct(
+            attribute::args_type(
+                *this,
+                token.ns,
+                nm,
+                ref_type
+            )
+        );
 
-        ret = attrs.back().get();
+        attrs.push_back(p);
+        ret.node = attrs.back();
     }
     else
     {
-        // Check if an element of the same name already exists.
-        element_store_type& children = *cur_element->child_elements;
-        auto it = std::find_if(
-            children.begin(), children.end(), find_by_name<element>(token.ns, token.name));
-        if (it == children.end())
-        {
-            // No element of that name exists.
-            children.push_back(
-                orcus::make_unique<element>(
-                    token.ns, m_names.intern(token.name.get(), token.name.size()).first,
-                    element_linked, ref_type));
+        element* elem = cur_element->get_or_create_linked_child(*this, token.ns, token.name, ref_type);
+        ret.elem_stack.push_back(elem);
+        ret.node = elem;
 
-            elem_stack_new.push_back(children.back().get());
-            ret = children.back().get();
-        }
-        else
-        {
-            // This element already exists.  Check if this is already linked.
-            element& elem = **it;
-            if (elem.ref_type != reference_unknown || elem.elem_type != element_unlinked)
-                throw xpath_error("This element is already linked.  You can't link the same element twice.");
-
-            // Turn this existing non-linked element into a linked one.
-            delete elem.child_elements;
-            elem.elem_type = element_linked;
-            elem.ref_type = ref_type;
-            switch (ref_type)
-            {
-                case reference_cell:
-                    elem.cell_ref = new cell_reference;
-                break;
-                case reference_range_field:
-                    elem.field_ref = new field_in_range;
-                break;
-                default:
-                    throw general_error("Unknown reference type in xml_map_tree::get_element_stack.");
-            }
-
-            elem_stack_new.push_back(&elem);
-            ret = &elem;
-        }
+        if (elem->row_group)
+            row_group_elem = elem;
     }
 
-    elem_stack.swap(elem_stack_new);
-
+    ret.anchor_elem = row_group_elem;
     return ret;
 }
 
-std::ostream& operator<< (std::ostream& os, const xml_map_tree::cell_position& ref)
+xml_map_tree::element* xml_map_tree::get_element(const pstring& xpath)
 {
-    os << "[sheet='" << ref.sheet << "' row=" << ref.row << " column=" << ref.col << "]";
-    return os;
+    assert(!xpath.empty());
+    xpath_parser parser(m_xmlns_cxt, xpath.get(), xpath.size());
+
+    // Get the root element first.
+    xpath_parser::token token = parser.next();
+    if (mp_root)
+    {
+        // Make sure the root element's names are the same.
+        if (mp_root->ns != token.ns || mp_root->name != token.name)
+            throw xpath_error("path begins with inconsistent root level name.");
+    }
+    else
+    {
+        // First time the root element is encountered.
+        if (token.attribute)
+            throw xpath_error("root element cannot be an attribute.");
+
+        auto const nm = m_names.intern(token.name).first; // work around LLVM < 7 libc++ bug
+        mp_root = m_element_pool.construct(
+            element::args_type(
+                *this,
+                token.ns,
+                nm,
+                element_unlinked,
+                reference_unknown
+            )
+        );
+    }
+
+    element* cur_element = mp_root;
+    assert(cur_element->child_elements);
+
+    for (token = parser.next(); !token.name.empty(); token = parser.next())
+    {
+        // Check if the current element contains a child element of the same name.
+        if (token.attribute)
+            throw xpath_error("attribute was not expected.");
+
+        cur_element = cur_element->get_or_create_child(*this, token.ns, token.name);
+    }
+
+    assert(cur_element);
+    return cur_element;
 }
 
 std::ostream& operator<< (std::ostream& os, const xml_map_tree::linkable& link)
@@ -764,17 +848,6 @@ std::ostream& operator<< (std::ostream& os, const xml_map_tree::linkable& link)
     return os;
 }
 
-bool operator< (const xml_map_tree::cell_position& left, const xml_map_tree::cell_position& right)
-{
-    if (left.sheet != right.sheet)
-        return left.sheet < right.sheet;
-
-    if (left.row != right.row)
-        return left.row < right.row;
-
-    return left.col < right.col;
-}
-
 }
 
 /* vim:set shiftwidth=4 softtabstop=4 expandtab: */
