@@ -202,9 +202,9 @@ public:
         m_attrs.push_back(xml_structure_tree::entity_name(attr.ns, attr.name));
     }
 
-    root* release_root_element()
+    std::unique_ptr<root> release_root_element()
     {
-        return mp_root.release();
+        return std::move(mp_root);
     }
 };
 
@@ -260,22 +260,17 @@ void print_scope(ostream& os, const scopes_type& scopes, const xmlns_context& cx
 
 }
 
-struct xml_structure_tree_impl
+struct xml_structure_tree::impl
 {
     string_pool m_pool;
     xmlns_context& m_xmlns_cxt;
-    root* mp_root;
+    std::unique_ptr<root> mp_root;
 
-    xml_structure_tree_impl(const xml_structure_tree_impl&) = delete;
-    xml_structure_tree_impl& operator=(const xml_structure_tree_impl&) = delete;
+    impl(const impl&) = delete;
+    impl& operator=(const impl&) = delete;
 
-    xml_structure_tree_impl(xmlns_context& xmlns_cxt) :
-        m_xmlns_cxt(xmlns_cxt), mp_root(nullptr) {}
-
-    ~xml_structure_tree_impl()
-    {
-        delete mp_root;
-    }
+    impl(xmlns_context& xmlns_cxt) : m_xmlns_cxt(xmlns_cxt) {}
+    ~impl() {}
 
     std::string get_element_str(const xml_structure_tree::entity_name& name) const
     {
@@ -289,15 +284,15 @@ struct xml_structure_tree_impl
 
 struct xml_structure_tree::walker_impl
 {
-    const xml_structure_tree_impl& m_parent_impl;
+    const xml_structure_tree::impl& m_parent_impl;
     root* mp_root; /// Root element of the authoritative tree.
     element_ref m_cur_elem;
     std::vector<element_ref> m_scopes;
 
     walker_impl& operator=(const walker_impl&) = delete;
 
-    walker_impl(const xml_structure_tree_impl& parent_impl) :
-        m_parent_impl(parent_impl), mp_root(parent_impl.mp_root) {}
+    walker_impl(const xml_structure_tree::impl& parent_impl) :
+        m_parent_impl(parent_impl), mp_root(parent_impl.mp_root.get()) {}
 
     walker_impl(const walker_impl& r) :
         m_parent_impl(r.m_parent_impl), mp_root(r.mp_root), m_cur_elem(r.m_cur_elem), m_scopes(r.m_scopes) {}
@@ -336,20 +331,17 @@ xml_structure_tree::element::element() :
 xml_structure_tree::element::element(const entity_name& _name, bool _repeat) :
     name(_name), repeat(_repeat) {}
 
-xml_structure_tree::walker::walker(const xml_structure_tree_impl& parent_impl) :
-    mp_impl(new walker_impl(parent_impl))
+xml_structure_tree::walker::walker(const xml_structure_tree::impl& parent_impl) :
+    mp_impl(orcus::make_unique<walker_impl>(parent_impl))
 {
 }
 
 xml_structure_tree::walker::walker(const walker& r) :
-    mp_impl(new walker_impl(*r.mp_impl))
+    mp_impl(orcus::make_unique<walker_impl>(*r.mp_impl))
 {
 }
 
-xml_structure_tree::walker::~walker()
-{
-    delete mp_impl;
-}
+xml_structure_tree::walker::~walker() {}
 
 xml_structure_tree::walker& xml_structure_tree::walker::operator= (const walker& r)
 {
@@ -494,12 +486,9 @@ xml_structure_tree::element xml_structure_tree::walker::select_by_path(const std
 }
 
 xml_structure_tree::xml_structure_tree(xmlns_context& xmlns_cxt) :
-    mp_impl(new xml_structure_tree_impl(xmlns_cxt)) {}
+    mp_impl(orcus::make_unique<impl>(xmlns_cxt)) {}
 
-xml_structure_tree::~xml_structure_tree()
-{
-    delete mp_impl;
-}
+xml_structure_tree::~xml_structure_tree() {}
 
 void xml_structure_tree::parse(const char* p, size_t n)
 {
