@@ -543,9 +543,14 @@ document_tree::document_tree(document_tree&& other) :
 document_tree::~document_tree() {}
 
 void document_tree::load(const std::string& strm)
+{
+    load(strm.data(), strm.size());
+}
+
+void document_tree::load(const char* p, size_t n)
 {
     handler hdl;
-    yaml_parser<handler> parser(strm.data(), strm.size(), hdl);
+    yaml_parser<handler> parser(p, n, hdl);
     parser.parse();
     hdl.swap(mp_impl->m_docs);
 }
@@ -580,7 +585,7 @@ bool needs_quoting(const std::string& s)
 {
     // See if it contains certain characters...
     for (auto it = s.begin(), ite = s.end(); it != ite; ++it)
-        if (is_in(*it, "#'"))
+        if (is_in(*it, ORCUS_ASCII("#'")))
             return true;
 
     // See if the whole string is parsed as a number.
@@ -764,16 +769,16 @@ void dump_json_node(std::ostringstream& os, const yaml_value& node, size_t scope
             size_t pos = 0;
             for (auto it = key_order.begin(), ite = key_order.end(); it != ite; ++it, ++pos)
             {
-                const yaml_value* key = it->get();
-                if (key->type != node_t::string)
+                const yaml_value* this_key = it->get();
+                if (this_key->type != node_t::string)
                     throw document_error("JSON doesn't support non-string key.");
 
-                auto val_pos = vals.find(key);
+                auto val_pos = vals.find(this_key);
                 assert(val_pos != vals.end());
 
                 dump_json_item(
                     os,
-                    &static_cast<const yaml_value_string*>(key)->value_string,
+                    &static_cast<const yaml_value_string*>(this_key)->value_string,
                     *val_pos->second, scope, pos < (n-1));
             }
 
