@@ -33,6 +33,7 @@
 
 using namespace orcus;
 using namespace orcus::spreadsheet;
+namespace ss = orcus::spreadsheet;
 using namespace std;
 
 namespace {
@@ -113,12 +114,13 @@ void test_xlsx_import()
         // Check that against known control.
         path = dir;
         path.append("check.txt");
-        string control = load_file_content(path.c_str());
+        file_content control(path.data());
 
         assert(!check.empty());
         assert(!control.empty());
 
-        pstring s1(&check[0], check.size()), s2(&control[0], control.size());
+        pstring s1(&check[0], check.size());
+        pstring s2 = control.str();
         assert(s1.trim() == s2.trim());
     }
 }
@@ -352,6 +354,49 @@ void test_xlsx_background_fill()
     }
 }
 
+void test_xlsx_number_format()
+{
+    pstring path(SRCDIR"/test/xlsx/number-format/date-time.xlsx");
+    std::unique_ptr<spreadsheet::document> doc = load_doc(path);
+
+    spreadsheet::sheet* sh = doc->get_sheet(0);
+    assert(sh);
+
+    struct check
+    {
+        ss::row_t row;
+        ss::col_t col;
+        pstring expected;
+    };
+
+    std::vector<check> checks =
+    {
+        { 1, 1, "[$-F800]dddd\\,\\ mmmm\\ dd\\,\\ yyyy" },
+        { 2, 1, "[ENG][$-409]mmmm\\ d\\,\\ yyyy;@" },
+        { 3, 1, "m/d/yy;@" },
+    };
+
+// TODO : We need to build our own internal number format code table for
+// xlsx.  Firstly, xlsx uses numFmtId explicitly to link between the xf and
+// the format string and the ID's are not sequential. Secondly, there is a
+// set of built-in format strings for ID < 164, and that information is not
+// stored in the file.
+#if 0
+    spreadsheet::styles& styles = doc->get_styles();
+
+    for (const check& c : checks)
+    {
+        size_t xf = sh->get_cell_format(c.row, c.col);
+        const spreadsheet::cell_format_t* cf = styles.get_cell_format(xf);
+        assert(cf);
+
+        const spreadsheet::number_format_t* nf = styles.get_number_format(cf->number_format);
+        assert(nf);
+        assert(nf->format_string == c.expected);
+    }
+#endif
+}
+
 void test_xlsx_text_alignment()
 {
     pstring path(SRCDIR"/test/xlsx/text-alignment/input.xlsx");
@@ -749,17 +794,19 @@ void test_xlsx_pivot_two_pivot_caches()
     assert(fld);
     assert(fld->name == "F1");
 
-    // This field should contain 4 string items 'A', 'B', 'C' and 'D'.
-    std::set<pivot_cache_item_t> expected =
     {
-        pivot_cache_item_t(ORCUS_ASCII("A")),
-        pivot_cache_item_t(ORCUS_ASCII("B")),
-        pivot_cache_item_t(ORCUS_ASCII("C")),
-        pivot_cache_item_t(ORCUS_ASCII("D")),
-    };
+        // This field should contain 4 string items 'A', 'B', 'C' and 'D'.
+        std::set<pivot_cache_item_t> expected =
+        {
+            pivot_cache_item_t(ORCUS_ASCII("A")),
+            pivot_cache_item_t(ORCUS_ASCII("B")),
+            pivot_cache_item_t(ORCUS_ASCII("C")),
+            pivot_cache_item_t(ORCUS_ASCII("D")),
+        };
 
-    std::set<pivot_cache_item_t> actual(fld->items.begin(), fld->items.end());
-    assert(actual == expected);
+        std::set<pivot_cache_item_t> actual(fld->items.begin(), fld->items.end());
+        assert(actual == expected);
+    }
 
     fld = cache->get_field(1);
     assert(fld);
@@ -794,17 +841,19 @@ void test_xlsx_pivot_two_pivot_caches()
     assert(fld);
     assert(fld->name == "F2");
 
-    expected =
     {
-        pivot_cache_item_t(ORCUS_ASCII("W")),
-        pivot_cache_item_t(ORCUS_ASCII("X")),
-        pivot_cache_item_t(ORCUS_ASCII("Y")),
-        pivot_cache_item_t(ORCUS_ASCII("Z")),
-    };
+        std::set<pivot_cache_item_t> expected =
+        {
+            pivot_cache_item_t(ORCUS_ASCII("W")),
+            pivot_cache_item_t(ORCUS_ASCII("X")),
+            pivot_cache_item_t(ORCUS_ASCII("Y")),
+            pivot_cache_item_t(ORCUS_ASCII("Z")),
+        };
 
-    actual.clear();
-    actual.insert(fld->items.begin(), fld->items.end());
-    assert(actual == expected);
+        std::set<pivot_cache_item_t> actual;
+        actual.insert(fld->items.begin(), fld->items.end());
+        assert(actual == expected);
+    }
 
     fld = cache->get_field(1);
     assert(fld);
@@ -855,19 +904,21 @@ void test_xlsx_pivot_mixed_type_field()
     assert(fld->min_value && fld->min_value == 1.0);
     assert(fld->max_value && fld->max_value == 2.0);
 
-    // This field should contain 3 string items 'A', 'B', 'C' and 2 numeric
-    // items 1 and 2.
-    std::set<pivot_cache_item_t> expected =
     {
-        pivot_cache_item_t(ORCUS_ASCII("A")),
-        pivot_cache_item_t(ORCUS_ASCII("B")),
-        pivot_cache_item_t(ORCUS_ASCII("C")),
-        pivot_cache_item_t(1.0),
-        pivot_cache_item_t(2.0),
-    };
+        // This field should contain 3 string items 'A', 'B', 'C' and 2 numeric
+        // items 1 and 2.
+        std::set<pivot_cache_item_t> expected =
+        {
+            pivot_cache_item_t(ORCUS_ASCII("A")),
+            pivot_cache_item_t(ORCUS_ASCII("B")),
+            pivot_cache_item_t(ORCUS_ASCII("C")),
+            pivot_cache_item_t(1.0),
+            pivot_cache_item_t(2.0),
+        };
 
-    std::set<pivot_cache_item_t> actual(fld->items.begin(), fld->items.end());
-    assert(actual == expected);
+        std::set<pivot_cache_item_t> actual(fld->items.begin(), fld->items.end());
+        assert(actual == expected);
+    }
 
     // 2nd field should be a nuemric field between 1.1 and 1.5.
     fld = cache->get_field(1);
@@ -906,22 +957,24 @@ void test_xlsx_pivot_mixed_type_field()
     assert(fld->min_value && fld->min_value == 1.0);
     assert(fld->max_value && fld->max_value == 5.0);
 
-    // This field should contain 3 string items 'A', 'B', 'C' and 4 numeric
-    // items 1, 2, 3.5 and 5.
-    expected =
     {
-        pivot_cache_item_t(ORCUS_ASCII("A")),
-        pivot_cache_item_t(ORCUS_ASCII("B")),
-        pivot_cache_item_t(ORCUS_ASCII("C")),
-        pivot_cache_item_t(1.0),
-        pivot_cache_item_t(2.0),
-        pivot_cache_item_t(3.5),
-        pivot_cache_item_t(5.0),
-    };
+        // This field should contain 3 string items 'A', 'B', 'C' and 4 numeric
+        // items 1, 2, 3.5 and 5.
+        std::set<pivot_cache_item_t> expected =
+        {
+            pivot_cache_item_t(ORCUS_ASCII("A")),
+            pivot_cache_item_t(ORCUS_ASCII("B")),
+            pivot_cache_item_t(ORCUS_ASCII("C")),
+            pivot_cache_item_t(1.0),
+            pivot_cache_item_t(2.0),
+            pivot_cache_item_t(3.5),
+            pivot_cache_item_t(5.0),
+        };
 
-    actual.clear();
-    actual.insert(fld->items.begin(), fld->items.end());
-    assert(actual == expected);
+        std::set<pivot_cache_item_t> actual;
+        actual.insert(fld->items.begin(), fld->items.end());
+        assert(actual == expected);
+    }
 
     // 2nd field
     fld = cache->get_field(1);
@@ -975,17 +1028,19 @@ void test_xlsx_pivot_group_field()
     assert(fld);
     assert(fld->name == "Key");
 
-    // This field should contain 4 string items 'A', 'B', 'C' and 'D'.
-    std::set<pivot_cache_item_t> expected =
     {
-        pivot_cache_item_t(ORCUS_ASCII("A")),
-        pivot_cache_item_t(ORCUS_ASCII("B")),
-        pivot_cache_item_t(ORCUS_ASCII("C")),
-        pivot_cache_item_t(ORCUS_ASCII("D")),
-    };
+        // This field should contain 4 string items 'A', 'B', 'C' and 'D'.
+        std::set<pivot_cache_item_t> expected =
+        {
+            pivot_cache_item_t(ORCUS_ASCII("A")),
+            pivot_cache_item_t(ORCUS_ASCII("B")),
+            pivot_cache_item_t(ORCUS_ASCII("C")),
+            pivot_cache_item_t(ORCUS_ASCII("D")),
+        };
 
-    std::set<pivot_cache_item_t> actual(fld->items.begin(), fld->items.end());
-    assert(actual == expected);
+        std::set<pivot_cache_item_t> actual(fld->items.begin(), fld->items.end());
+        assert(actual == expected);
+    }
 
     // 2nd field is 'Value' and is a numeric field.
     fld = cache->get_field(1);
@@ -1009,16 +1064,18 @@ void test_xlsx_pivot_group_field()
     assert(gd->base_field == 0);
     assert(gd->items.size() == 2);
 
-    // It should have two items - Group1 and Group2.
-    expected =
     {
-        pivot_cache_item_t(ORCUS_ASCII("Group1")),
-        pivot_cache_item_t(ORCUS_ASCII("Group2")),
-    };
+        // It should have two items - Group1 and Group2.
+        std::set<pivot_cache_item_t> expected =
+        {
+            pivot_cache_item_t(ORCUS_ASCII("Group1")),
+            pivot_cache_item_t(ORCUS_ASCII("Group2")),
+        };
 
-    actual.clear();
-    actual.insert(gd->items.begin(), gd->items.end());
-    assert(actual == expected);
+        std::set<pivot_cache_item_t> actual;
+        actual.insert(gd->items.begin(), gd->items.end());
+        assert(actual == expected);
+    }
 
     // Group1 should group 'A' and 'B' from the 1st field, and Group2 should
     // group 'C' and 'D'.
@@ -1564,6 +1621,7 @@ int main()
     test_xlsx_merged_cells();
     test_xlsx_date_time();
     test_xlsx_background_fill();
+    test_xlsx_number_format();
     test_xlsx_text_alignment();
     test_xlsx_cell_borders_single_cells();
     test_xlsx_cell_borders_directions();
