@@ -12,18 +12,29 @@
 
 #include "orcus_filter_global.hpp"
 
+#include <iostream>
+
 using namespace orcus;
 
 int main(int argc, char** argv)
 {
-    spreadsheet::document doc;
-    spreadsheet::view view(doc);
-    spreadsheet::import_factory fact(doc, view);
-    orcus_xlsx app(&fact);
+    try
+    {
+        spreadsheet::document doc;
+        spreadsheet::view view(doc);
+        spreadsheet::import_factory fact(doc, view);
+        orcus_xlsx app(&fact);
 
-    if (parse_import_filter_args(argc, argv, fact, app, doc))
+        if (parse_import_filter_args(argc, argv, fact, app, doc))
+            return EXIT_FAILURE;
+    }
+    catch (const std::exception& e)
+    {
+        std::cerr << e.what() << std::endl;
         return EXIT_FAILURE;
+    }
 
     return EXIT_SUCCESS;
 }
+
 /* vim:set shiftwidth=4 softtabstop=4 expandtab: */
