@@ -6,6 +6,9 @@
  */
 
 #include "orcus/csv_parser_base.hpp"
+#include "orcus/global.hpp"
+
+#include <cstring>
 
 namespace orcus { namespace csv {
 
@@ -24,11 +27,14 @@ const char* parse_error::what() const throw()
 
 parser_base::parser_base(
     const char* p, size_t n, const csv::parser_config& config) :
-    ::orcus::parser_base(p, n), m_config(config) {}
+    ::orcus::parser_base(p, n, false), m_config(config)
+{
+    maybe_skip_bom();
+}
 
 bool parser_base::is_blank(char c) const
 {
-    return is_in(c, " \t");
+    return is_in(c, ORCUS_ASCII(" \t"));
 }
 
 bool parser_base::is_delim(char c) const
@@ -43,9 +49,20 @@ bool parser_base::is_text_qualifier(char c) const
 
 void parser_base::skip_blanks()
 {
-    skip(" \t");
+    skip(ORCUS_ASCII(" \t"));
 }
 
+void parser_base::maybe_skip_bom()
+{
+    if (remaining_size() < 3)
+        // Ensure that the stream contains at least 3 bytes.
+        return;
+
+    static const char* bom = "\xEF\xBB\xBF";
+    if (!strncmp(mp_char, bom, 3))
+        next(3);
+}
 
 }}
+
 /* vim:set shiftwidth=4 softtabstop=4 expandtab: */
