@@ -5,13 +5,11 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  */
 
+#include "test_global.hpp"
 #include "orcus/string_pool.hpp"
 #include "orcus/pstring.hpp"
 #include "orcus/global.hpp"
 
-#include <iostream>
-#include <cassert>
-
 using namespace std;
 using namespace orcus;
 
