@@ -15,52 +15,29 @@
 #include "orcus/global.hpp"
 #endif
 
-#include <iostream>
-
-using namespace std;
-
 namespace orcus { namespace python {
 
 #ifdef __ORCUS_PYTHON_XLSX
 
-PyObject* xlsx_read_file(PyObject* /*module*/, PyObject* args, PyObject* kwargs)
+PyObject* xlsx_read(PyObject* /*module*/, PyObject* args, PyObject* kwargs)
 {
-    static const char* kwlist[] = { "filepath", nullptr };
-
-    const char* filepath = nullptr;
-
-    if (!PyArg_ParseTupleAndKeywords(args, kwargs, "s", const_cast<char**>(kwlist), &filepath))
+    PyObject* obj_bytes = read_stream_object_from_args(args, kwargs);
+    if (!obj_bytes)
         return nullptr;
 
     std::unique_ptr<spreadsheet::document> doc = orcus::make_unique<spreadsheet::document>();
     spreadsheet::import_factory fact(*doc);
     orcus_xlsx app(&fact);
 
-    app.read_file(filepath);
-
-    PyTypeObject* doc_type = get_document_type();
-    if (!doc_type)
-        return nullptr;
-
-    PyObject* obj_doc = doc_type->tp_new(doc_type, nullptr, nullptr);
-    if (!obj_doc)
-        return nullptr;
-
-    doc_type->tp_init(obj_doc, nullptr, nullptr);
-
-    store_document(obj_doc, std::move(doc));
-
-    Py_INCREF(obj_doc);
-    return obj_doc;
+    return import_from_stream_into_document(obj_bytes, app, std::move(doc));
 }
 
 #else
 
-PyObject* xlsx_read_file(PyObject*, PyObject*, PyObject*)
+PyObject* xlsx_read(PyObject*, PyObject*, PyObject*)
 {
-    // TODO : raise a python exception here.
-    Py_INCREF(Py_None);
-    return Py_None;
+    PyErr_SetString(PyExc_RuntimeError, "The xlsx module is not enabled.");
+    return nullptr;
 }
 
 #endif
