@@ -26,12 +26,15 @@
 #include <ixion/formula_name_resolver.hpp>
 #include <ixion/interface/table_handler.hpp>
 #include <ixion/config.hpp>
+#include <boost/filesystem.hpp>
 
 #include <iostream>
 #include <fstream>
+#include <sstream>
 #include <map>
 
 using namespace std;
+namespace fs = boost::filesystem;
 
 namespace orcus { namespace spreadsheet {
 
@@ -286,7 +289,7 @@ struct document_impl
     sheet_items_type m_sheets;
     styles m_styles;
     import_shared_strings* mp_strings;
-    ixion::dirty_formula_cells_t m_dirty_cells;
+    ixion::abs_range_set_t m_dirty_cells;
 
     pivot_collection m_pivots;
 
@@ -302,7 +305,7 @@ struct document_impl
         mp_strings(new import_shared_strings(m_string_pool, m_context, m_styles)),
         m_pivots(doc),
         mp_name_resolver(ixion::formula_name_resolver::get(ixion::formula_name_resolver_t::excel_a1, &m_context)),
-        m_grammar(formula_grammar_t::xlsx_2007),
+        m_grammar(formula_grammar_t::xlsx),
         m_table_handler(m_context, m_tables)
     {
         m_context.set_table_handler(&m_table_handler);
@@ -452,8 +455,12 @@ const sheet* document::get_sheet(sheet_t sheet_pos) const
 
 void document::calc_formulas()
 {
+    ixion::abs_range_set_t empty;
+
     ixion::model_context& cxt = get_model_context();
-    ixion::calculate_cells(cxt, mp_impl->m_dirty_cells, 0);
+    std::vector<ixion::abs_range_t> sorted = ixion::query_and_sort_dirty_cells(
+        cxt, empty, &mp_impl->m_dirty_cells);
+    ixion::calculate_sorted_cells(cxt, sorted, 0);
 }
 
 void document::clear()
@@ -461,6 +468,72 @@ void document::clear()
     mp_impl.reset(new document_impl(*this));
 }
 
+void document::dump(dump_format_t format, const std::string& output) const
+{
+    if (format == dump_format_t::none)
+        return;
+
+    if (format == dump_format_t::check)
+    {
+        // For this output, we write to a single file.
+        std::ostream* ostrm = &std::cout;
+        std::unique_ptr<std::ofstream> fs;
+
+        if (!output.empty())
+        {
+            if (fs::is_directory(output))
+            {
+                std::ostringstream os;
+                os << "Output file path points to an existing directory.";
+                throw std::invalid_argument(os.str());
+            }
+
+            // Output to stdout when output path is not given.
+            fs = orcus::make_unique<std::ofstream>(output.data());
+            ostrm = fs.get();
+        }
+
+        dump_check(*ostrm);
+        return;
+    }
+
+    if (output.empty())
+        throw std::invalid_argument("No output directory.");
+
+    if (fs::exists(output))
+    {
+        if (!fs::is_directory(output))
+        {
+            std::ostringstream os;
+            os << "A file named '" << output << "' already exists, and is not a directory.";
+            throw std::invalid_argument(os.str());
+        }
+    }
+    else
+        fs::create_directory(output);
+
+    switch (format)
+    {
+        case dump_format_t::csv:
+            dump_csv(output);
+            break;
+        case dump_format_t::flat:
+            dump_flat(output);
+            break;
+        case dump_format_t::html:
+            dump_html(output);
+            break;
+        case dump_format_t::json:
+            dump_json(output);
+            break;
+        case dump_format_t::none:
+        case dump_format_t::unknown:
+            break;
+        default:
+            ;
+    }
+}
+
 void document::dump_flat(const string& outdir) const
 {
     cout << "----------------------------------------------------------------------" << endl;
@@ -614,8 +687,7 @@ void document::set_formula_grammar(formula_grammar_t grammar)
             resolver_type = ixion::formula_name_resolver_t::excel_r1c1;
             arg_sep = ',';
             break;
-        case formula_grammar_t::xlsx_2007:
-        case formula_grammar_t::xlsx_2010:
+        case formula_grammar_t::xlsx:
             resolver_type = ixion::formula_name_resolver_t::excel_a1;
             arg_sep = ',';
             break;
